/*  Autopackage GTK+ frontend
 *  Copyright (C) 2003  Hongli Lai <h.lai@chello.nl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _RECEIVER_C_
#define _RECEIVER_C_

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include "receiver.h"
#include "main.h"
#include "protocol.h"
#include "utils.h"


static GThread *receiver_thread = NULL;
static gboolean stop = FALSE;
gboolean readLock = FALSE;
gboolean pokeFifo = TRUE;


static gpointer
read_data (gpointer d)
{
	int writeFD = GPOINTER_TO_INT (d);

	while (!stop)
	{
		gchar *data;

		while (readLock) usleep (1000);
		if (stop) break;

		data = protocol_recv ();
		if (!data) break;
		if (stop)
		{
			g_free (data);
			break;
		}

		readLock = TRUE;
		write (writeFD, data, strlen (data));
		write (writeFD, "-\n", strlen ("-\n"));
		g_free (data);
	}
	return NULL;
}


/* Start a receiver thread and create an IOChannel for reading input. */
GIOChannel *
receiver_start (GError **err)
{
	GIOChannel *io;
	int p[2];

	g_return_val_if_fail (receiver_thread == NULL, NULL);

	if (pipe (p) != 0) return NULL;
	io = g_io_channel_unix_new (p[0]);
	if (!io) return NULL;
	g_io_channel_set_encoding (io, NULL, NULL);
	g_io_channel_set_close_on_unref (io, TRUE);

	receiver_thread = g_thread_create (read_data, GINT_TO_POINTER (p[1]),
		FALSE, err);
	return io;
}


void
receiver_stop (void)
{
	int fd, fd2;

	if (!receiver_thread)
		return;

	stop = TRUE;
	receiver_thread = NULL;
	readLock = FALSE;

	if (!pokeFifo) return;

	/* Touch and remove the FIFO to prevent the backend from freezing */
	fd = open (fifoName, O_RDONLY | O_NONBLOCK | O_NOCTTY,
		S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
	fd2 = open (fifoName, O_WRONLY | O_NONBLOCK | O_NOCTTY,
		S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
	remove (fifoName);
	if (fd != -1) close (fd);
	if (fd2 != -1) close (fd2);
}


#endif /* _RECEIVER_C_ */
